package list

import (
	"strings"

	"github.com/k0kubun/pp/v3"
	"github.com/spf13/cobra"

	"gitea.libretechconsulting.com/rmcguire/eia-api-go/cmd/eia-client/internal/util"
	"gitea.libretechconsulting.com/rmcguire/eia-api-go/pkg/eia"
)

var listRoutesCmd = &cobra.Command{
	Use:     "routes",
	Aliases: []string{"methods", "r"},
	Short:   "List routes, optionally by type (default is Data)",
	Args:    cobra.MaximumNArgs(2),
	ValidArgs: []string{
		"Data",
		"Facet",
		"FacetId",
	},
	Run: RunListRoutesCmd,
}

const (
	allMethodsFlag  = "all"
	filtersFlag     = "filters"
	routePrefixFlag = "routePrefix"
)

var defaultExcludedSuffixes = []string{
	"WithBody",
	"Facet",
	"FacetId",
}

func RunListRoutesCmd(cmd *cobra.Command, args []string) {
	logger := util.Logger(cmd)

	// Command flags
	allRoutes, _ := cmd.Flags().GetBool(allMethodsFlag)
	routePrefix, _ := cmd.Flags().GetString(routePrefixFlag)
	userFilters, _ := cmd.Flags().GetStringSlice(filtersFlag)

	if allRoutes && len(args) > 0 {
		logger.Fatal().Msg("can't specify all methods while also filtering by type")
	}

	typeFilter := make([]string, 0)
	if !allRoutes && len(args) == 1 {
		typeFilter = append(typeFilter, args[0])
	}

	routes := eia.GetRoutes(typeFilter...)
	filteredRoutes := make([]string, 0, len(routes))

	for _, route := range routes {
		// Filter prefixes
		if !allRoutes && routePrefix != "" {
			if !strings.HasPrefix(strings.ToLower(route), strings.ToLower(routePrefix)) {
				continue
			}
		}

		// Default exclusions
		if !allRoutes && len(typeFilter) < 1 && len(userFilters) < 1 {
			for _, e := range defaultExcludedSuffixes {
				if strings.HasSuffix(strings.ToLower(route), strings.ToLower(e)) {
					goto next
				}
			}
		}

		// User inclusion filters
		if len(userFilters) > 0 {
			for _, f := range userFilters {
				if strings.Contains(strings.ToLower(route), strings.ToLower(f)) {
					filteredRoutes = append(filteredRoutes, route)
				}
			}
		} else {
			filteredRoutes = append(filteredRoutes, route)
		}
	next:
	}

	pp.Println(filteredRoutes)
}

func init() {
	listRoutesCmd.PersistentFlags().BoolP(allMethodsFlag, "a", false, "List all methods, no filtering")
	listRoutesCmd.PersistentFlags().StringSliceP(filtersFlag, "f", []string{}, "Optional [inclusion] filters, case insensitive")
	listRoutesCmd.PersistentFlags().StringP(routePrefixFlag, "p", "Get", "Prefix for routes, ignore with -a flag")
}
